@@ -40,6 +40,12 @@
 
 #include <string.h>
 
+/* Maemo port */
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include "dbus-server.h"
+#include "dbus-bindings.h"
+#include "mce/dbus-names.h"
+
 #define PLUGIN_ID "pidgin-libnotify"
 
 static GHashTable *buddy_hash;
@@ -52,6 +58,21 @@ get_plugin_pref_frame (PurplePlugin *plugin)
 
 	frame = purple_plugin_pref_frame_new ();
 
+	ppref = purple_plugin_pref_new_with_name_and_label (
+                            "/plugins/gtk/libnotify/autoclose",
+                            _("Autoclose notification"));
+	purple_plugin_pref_frame_add (frame, ppref);
+
+	ppref = purple_plugin_pref_new_with_name_and_label (
+                            "/plugins/gtk/libnotify/vibrate",
+                            _("Vibrate"));
+	purple_plugin_pref_frame_add (frame, ppref);
+
+	ppref = purple_plugin_pref_new_with_name_and_label (
+                            "/plugins/gtk/libnotify/led",
+                            _("Led notification"));
+	purple_plugin_pref_frame_add (frame, ppref);
+
 	ppref = purple_plugin_pref_new_with_name_and_label (
                             "/plugins/gtk/libnotify/newmsg",
                             _("New messages"));
@@ -143,6 +164,8 @@ best_name (PurpleBuddy *buddy)
 	}
 }
 
+/* Not needed in Maemo */
+/*
 static GdkPixbuf *
 pixbuf_from_buddy_icon (PurpleBuddyIcon *buddy_icon)
 {
@@ -168,6 +191,55 @@ pixbuf_from_buddy_icon (PurpleBuddyIcon *buddy_icon)
 
 	return icon;
 }
+*/
+
+static void
+notify_send_dbus_mce_message (const gchar *interface,
+			      const gchar *pattern)
+{
+	DBusMessage *message;
+	DBusError error;
+	DBusConnection *connection;
+
+	purple_debug_info (PLUGIN_ID, "notify_send_dbus_mce_message(), "
+					"interface: %s, pattern: '%s'\n", interface, pattern);
+
+	dbus_error_init(&error);
+	connection = dbus_bus_get (DBUS_MESSAGE_TYPE_METHOD_CALL, &error);
+
+	/* Create the message we need */
+	message = dbus_message_new_method_call( MCE_SERVICE, 
+						MCE_REQUEST_PATH, 
+						MCE_REQUEST_IF, 
+						interface);
+
+	/* Append the Vibration Pattern to the signal */
+	dbus_message_append_args(message,
+				 DBUS_TYPE_STRING, &pattern,
+				 DBUS_TYPE_INVALID);
+
+	/* Send the signal */
+
+	purple_debug_info (PLUGIN_ID, "notify_send_dbus_mce_message(), sending dbus mce message\n");
+	if (!dbus_connection_send(connection, message, NULL)) {
+		purple_debug_error (PLUGIN_ID, "notify_send_dbus_mce_message(), failed to send dbus message\n");
+	}
+
+	if (dbus_error_is_set (&error)) {
+		purple_debug_error (PLUGIN_ID, 
+				    "notify_send_dbus_mce_message(), failed to send dbus message: %s: %s\n",
+				    error.name,
+				    error.message);
+	}
+
+	/* Flush the queue */
+	dbus_connection_flush (connection);
+
+	/* Free the message & connection now we have finished with it */
+	dbus_message_unref (message);
+	dbus_connection_unref (connection);
+}
+
 
 static void
 action_cb (NotifyNotification *notification,
@@ -177,7 +249,7 @@ action_cb (NotifyNotification *notification,
 	PurpleConversation *conv = NULL;
 
 	purple_debug_info (PLUGIN_ID, "action_cb(), "
-					"notification: 0x%x, action: '%s'", notification, action);
+					"notification: 0x%x, action: '%s'", (unsigned int) notification, action);
 
 	buddy = (PurpleBuddy *)g_object_get_data (G_OBJECT(notification), "buddy");
 
@@ -201,13 +273,23 @@ action_cb (NotifyNotification *notification,
 static gboolean
 closed_cb (NotifyNotification *notification)
 {
+	PurpleBuddy *buddy;
 	PurpleContact *contact;
 
-	purple_debug_info (PLUGIN_ID, "closed_cb(), notification: 0x%x\n", notification);
+	purple_debug_info (PLUGIN_ID, "closed_cb(), notification: 0x%x\n", (unsigned int) notification);
 
-	contact = (PurpleContact *)g_object_get_data (G_OBJECT(notification), "contact");
-	if (contact)
-		g_hash_table_remove (buddy_hash, contact);
+	// contact = (PurpleContact *)g_object_get_data (G_OBJECT(notification), "contact");
+	buddy = (PurpleBuddy *)g_object_get_data (G_OBJECT(notification), "buddy");
+	if (!buddy) {
+		purple_debug_warning (PLUGIN_ID, "Got no buddy!");
+	} else {
+		contact = purple_buddy_get_contact (buddy);
+		if (contact)
+			g_hash_table_remove (buddy_hash, contact);
+	}
+
+	if (purple_prefs_get_bool ("/plugins/gtk/libnotify/led"))
+	       notify_send_dbus_mce_message(MCE_DEACTIVATE_LED_PATTERN,"PatternCommunicationIM");
 
 	g_object_unref (G_OBJECT(notification));
 
@@ -260,8 +342,8 @@ notify (const gchar *title,
 		PurpleBuddy *buddy)
 {
 	NotifyNotification *notification = NULL;
-	GdkPixbuf *icon;
-	PurpleBuddyIcon *buddy_icon;
+	// GdkPixbuf *icon;
+	// PurpleBuddyIcon *buddy_icon;
 	gchar *tr_body;
 	PurpleContact *contact;
 
@@ -275,7 +357,7 @@ notify (const gchar *title,
 	notification = g_hash_table_lookup (buddy_hash, contact);
 
 	if (notification != NULL) {
-		notify_notification_update (notification, title, tr_body, NULL);
+		notify_notification_update (notification, title, tr_body, "pidgin");
 		/* this shouldn't be necessary, file a bug */
 		notify_notification_show (notification, NULL);
 
@@ -286,13 +368,15 @@ notify (const gchar *title,
 		g_free (tr_body);
 		return;
 	}
-	notification = notify_notification_new (title, tr_body, NULL, NULL);
+	notification = notify_notification_new (title, tr_body, "pidgin", NULL);
 	purple_debug_info (PLUGIN_ID, "notify(), new: "
 					 "title: '%s', body: '%s', buddy: '%s'\n",
 					 title, tr_body, best_name (buddy));
 
 	g_free (tr_body);
 
+	/* Custom Icons doesn't work in Maemo. Only Theme icons work */
+	/*
 	buddy_icon = purple_buddy_get_icon (buddy);
 	if (buddy_icon) {
 		icon = pixbuf_from_buddy_icon (buddy_icon);
@@ -308,26 +392,38 @@ notify (const gchar *title,
 	} else {
 		purple_debug_warning (PLUGIN_ID, "notify(), couldn't find any icon!\n");
 	}
+	*/
 
 	g_hash_table_insert (buddy_hash, contact, notification);
 
-	g_object_set_data (G_OBJECT(notification), "contact", contact);
+	// g_object_set_data (G_OBJECT(notification), "contact", contact);
+	g_object_set_data (G_OBJECT(notification), "buddy", buddy);
 
 	g_signal_connect (notification, "closed", G_CALLBACK(closed_cb), NULL);
 
 	notify_notification_set_urgency (notification, NOTIFY_URGENCY_NORMAL);
 
-	notify_notification_add_action (notification, "show", _("Show"), action_cb, NULL, NULL);
+	if (purple_prefs_get_bool ("/plugins/gtk/libnotify/autoclose"))
+		notify_notification_set_timeout (notification, 3000);
+
+	// notify_notification_add_action (notification, "show", _("Show"), action_cb, NULL, NULL);
+	notify_notification_add_action (notification, "default", _("Show"), action_cb, NULL, NULL);
 
 	if (!notify_notification_show (notification, NULL)) {
 		purple_debug_error (PLUGIN_ID, "notify(), failed to send notification\n");
 	}
 
+	/* Maemo port */
+	if (purple_prefs_get_bool ("/plugins/gtk/libnotify/vibrate"))
+	       notify_send_dbus_mce_message(MCE_ACTIVATE_VIBRATOR_PATTERN,"PatternChatAndEmail");
+
+	if (purple_prefs_get_bool ("/plugins/gtk/libnotify/led"))
+	       notify_send_dbus_mce_message(MCE_ACTIVATE_LED_PATTERN,"PatternCommunicationIM");
 }
 
 static void
 notify_buddy_signon_cb (PurpleBuddy *buddy,
-						gpointer data)
+			gpointer data)
 {
 	gchar *tr_name, *title;
 	gboolean blocked;
@@ -434,13 +530,13 @@ notify_new_message_cb (PurpleAccount *account,
 
 #ifndef DEBUG /* in debug mode, always show notifications */
 	if (conv && purple_conversation_has_focus (conv)) {
-		purple_debug_info (PLUGIN_ID, "Conversation has focus 0x%x\n", conv);
+		purple_debug_info (PLUGIN_ID, "Conversation has focus 0x%x\n", (unsigned int) conv);
 		return;
 	}
 #endif
 
 	if (conv && purple_prefs_get_bool ("/plugins/gtk/libnotify/newconvonly")) {
-		purple_debug_info (PLUGIN_ID, "Conversation is not new 0x%x\n", conv);
+		purple_debug_info (PLUGIN_ID, "Conversation is not new 0x%x\n", (unsigned int) conv);
 		return;
 	}
 
@@ -542,30 +638,30 @@ static PurplePluginUiInfo prefs_info = {
 };
 
 static PurplePluginInfo info = {
-    PURPLE_PLUGIN_MAGIC,										/* api version */
+    PURPLE_PLUGIN_MAGIC,	/* api version */
     PURPLE_MAJOR_VERSION,
     PURPLE_MINOR_VERSION,
-    PURPLE_PLUGIN_STANDARD,									/* type */
-    0,														/* ui requirement */
-    0,														/* flags */
-    NULL,													/* dependencies */
-    PURPLE_PRIORITY_DEFAULT,									/* priority */
+    PURPLE_PLUGIN_STANDARD,	/* type */
+    0,				/* ui requirement */
+    0,				/* flags */
+    NULL,			/* dependencies */
+    PURPLE_PRIORITY_DEFAULT,	/* priority */
     
-    PLUGIN_ID,												/* id */
-    NULL,													/* name */
-    VERSION,												/* version */
-    NULL,													/* summary */
-    NULL,													/* description */
+    PLUGIN_ID,			/* id */
+    NULL,			/* name */
+    VERSION,			/* version */
+    NULL,			/* summary */
+    NULL,			/* description */
     
-    "Duarte Henriques <duarte.henriques@gmail.com>",		/* author */
+    "Duarte Henriques <duarte.henriques@gmail.com>, Naikel Aparicio <naikel@gmail.com>",	/* author */
     "http://sourceforge.net/projects/gaim-libnotify/",		/* homepage */
     
-    plugin_load,			/* load */
-    plugin_unload,			/* unload */
-    NULL,					/* destroy */
-    NULL,					/* ui info */
-    NULL,					/* extra info */
-    &prefs_info				/* prefs info */
+    plugin_load,		/* load */
+    plugin_unload,		/* unload */
+    NULL,			/* destroy */
+    NULL,			/* ui info */
+    NULL,			/* extra info */
+    &prefs_info			/* prefs info */
 };
 
 static void
@@ -579,6 +675,9 @@ init_plugin (PurplePlugin *plugin)
 	info.description = _("Pidgin-libnotify:\nDisplays popups via libnotify.");
 
 	purple_prefs_add_none ("/plugins/gtk/libnotify");
+	purple_prefs_add_bool ("/plugins/gtk/libnotify/autoclose", TRUE);
+	purple_prefs_add_bool ("/plugins/gtk/libnotify/vibrate", TRUE);
+	purple_prefs_add_bool ("/plugins/gtk/libnotify/led", TRUE);
 	purple_prefs_add_bool ("/plugins/gtk/libnotify/newmsg", TRUE);
 	purple_prefs_add_bool ("/plugins/gtk/libnotify/blocked", TRUE);
 	purple_prefs_add_bool ("/plugins/gtk/libnotify/newconvonly", FALSE);
